@@ -1,12 +1,11 @@
 /* Redmine - project management software
-   Copyright (C) 2006-2017  Jean-Philippe Lang */
+   Copyright (C) 2006-2022  Jean-Philippe Lang */
 
-/* Fix for CVE-2015-9251, to be removed with JQuery >= 3.0 */
-$.ajaxPrefilter(function (s) {
-  if (s.crossDomain) {
-    s.contents.script = false;
-  }
-});
+function sanitizeHTML(string) {
+  var temp = document.createElement('span');
+  temp.textContent = string;
+  return temp.innerHTML;
+}
 
 function checkAll(id, checked) {
   $('#'+id).find('input[type=checkbox]:enabled').prop('checked', checked);
@@ -32,6 +31,7 @@ function toggleRowGroup(el) {
   var tr = $(el).parents('tr').first();
   var n = tr.next();
   tr.toggleClass('open');
+  $(el).toggleClass('icon-expanded icon-collapsed');
   while (n.length && !n.hasClass('group')) {
     n.toggle();
     n = n.next('tr');
@@ -43,6 +43,7 @@ function collapseAllRowGroups(el) {
   tbody.children('tr').each(function(index) {
     if ($(this).hasClass('group')) {
       $(this).removeClass('open');
+      $(this).find('.expander').switchClass('icon-expanded', 'icon-collapsed');
     } else {
       $(this).hide();
     }
@@ -54,6 +55,7 @@ function expandAllRowGroups(el) {
   tbody.children('tr').each(function(index) {
     if ($(this).hasClass('group')) {
       $(this).addClass('open');
+      $(this).find('.expander').switchClass('icon-collapsed', 'icon-expanded');
     } else {
       $(this).show();
     }
@@ -72,6 +74,7 @@ function toggleAllRowGroups(el) {
 function toggleFieldset(el) {
   var fieldset = $(el).parents('fieldset').first();
   fieldset.toggleClass('collapsed');
+  fieldset.children('legend').toggleClass('icon-expanded icon-collapsed');
   fieldset.children('div').toggle();
 }
 
@@ -116,9 +119,6 @@ function initFilters() {
   $('#filters-table').on('click', 'td.field input[type=checkbox]', function() {
     toggleFilter($(this).val());
   });
-  $('#filters-table').on('click', '.toggle-multiselect', function() {
-    toggleMultiSelect($(this).siblings('select'));
-  });
   $('#filters-table').on('keypress', 'input[type=text]', function(e) {
     if (e.keyCode == 13) $(this).closest('form').submit();
   });
@@ -146,6 +146,7 @@ function addFilter(field, operator, values) {
   }
   $('#cb_'+fieldId).prop('checked', true);
   toggleFilter(field);
+  toggleMultiSelectIconInit();
   $('#add_filter_select').val('').find('option').each(function() {
     if ($(this).attr('value') == field) {
       $(this).attr('disabled', true);
@@ -172,7 +173,7 @@ function buildFilterRow(field, operator, values) {
   select = tr.find('td.operator select');
   for (i = 0; i < operators.length; i++) {
     var option = $('<option>').val(operators[i]).text(operatorLabels[operators[i]]);
-    if (operators[i] == operator) { option.attr('selected', true); }
+    if (operators[i] == operator) { option.prop('selected', true); }
     select.append(option);
   }
   select.change(function(){ toggleOperator(field); });
@@ -184,7 +185,7 @@ function buildFilterRow(field, operator, values) {
   case "list_subprojects":
     tr.find('td.values').append(
       '<span style="display:none;"><select class="value" id="values_'+fieldId+'_1" name="v['+field+'][]"></select>' +
-      ' <span class="toggle-multiselect">&nbsp;</span></span>'
+      ' <span class="toggle-multiselect icon-only '+(values.length > 1 ? 'icon-toggle-minus' : 'icon-toggle-plus')+'">&nbsp;</span></span>'
     );
     select = tr.find('td.values select');
     if (values.length > 1) { select.attr('multiple', true); }
@@ -193,7 +194,7 @@ function buildFilterRow(field, operator, values) {
       var option = $('<option>');
       if ($.isArray(filterValue)) {
         option.val(filterValue[1]).text(filterValue[0]);
-        if ($.inArray(filterValue[1], values) > -1) {option.attr('selected', true);}
+        if ($.inArray(filterValue[1], values) > -1) {option.prop('selected', true);}
         if (filterValue.length == 3) {
           var optgroup = select.find('optgroup').filter(function(){return $(this).attr('label') == filterValue[2]});
           if (!optgroup.length) {optgroup = $('<optgroup>').attr('label', filterValue[2]);}
@@ -201,7 +202,7 @@ function buildFilterRow(field, operator, values) {
         }
       } else {
         option.val(filterValue).text(filterValue);
-        if ($.inArray(filterValue, values) > -1) {option.attr('selected', true);}
+        if ($.inArray(filterValue, values) > -1) {option.prop('selected', true);}
       }
       select.append(option);
     }
@@ -235,7 +236,7 @@ function buildFilterRow(field, operator, values) {
       var filterValue = filterValues[i];
       var option = $('<option>');
       option.val(filterValue[1]).text(filterValue[0]);
-      if (values[0] == filterValue[1]) { option.attr('selected', true); }
+      if (values[0] == filterValue[1]) { option.prop('selected', true); }
       select.append(option);
     }
     break;
@@ -289,11 +290,14 @@ function toggleOperator(field) {
   switch (operator.val()) {
     case "!*":
     case "*":
+    case "nd":
     case "t":
     case "ld":
+    case "nw":
     case "w":
     case "lw":
     case "l2w":
+    case "nm":
     case "m":
     case "lm":
     case "y":
@@ -328,11 +332,14 @@ function toggleOperator(field) {
 }
 
 function toggleMultiSelect(el) {
+  var isWorkflow = el.closest('.controller-workflows');
   if (el.attr('multiple')) {
     el.removeAttr('multiple');
+    if (isWorkflow) { el.find("option[value=all]").show(); }
     el.attr('size', 1);
   } else {
     el.attr('multiple', true);
+    if (isWorkflow) { el.find("option[value=all]").attr("selected", false).hide(); }
     if (el.children().length > 10)
       el.attr('size', 10);
     else
@@ -345,12 +352,82 @@ function showTab(name, url) {
   $('#tab-content-' + name).show();
   $('#tab-' + name).closest('.tabs').find('a').removeClass('selected');
   $('#tab-' + name).addClass('selected');
-  //replaces current URL with the "href" attribute of the current link
-  //(only triggered if supported by browser)
-  if ("replaceState" in window.history) {
+
+  replaceInHistory(url)
+
+  return false;
+}
+
+function showIssueHistory(journal, url) {
+  tab_content = $('#tab-content-history');
+  tab_content.parent().find('.tab-content').hide();
+  tab_content.show();
+  tab_content.parent().children('div.tabs').find('a').removeClass('selected');
+
+  $('#tab-' + journal).addClass('selected');
+
+  replaceInHistory(url)
+
+  switch(journal) {
+    case 'notes':
+      tab_content.find('.journal').show();
+      tab_content.find('.journal:not(.has-notes)').hide();
+      tab_content.find('.journal .wiki').show();
+      tab_content.find('.journal .contextual .journal-actions').show();
+
+      // always show thumbnails in notes tab
+      var thumbnails = tab_content.find('.journal .thumbnails');
+      thumbnails.show();
+      // show journals without notes, but with thumbnails
+      thumbnails.parents('.journal').show();
+      break;
+    case 'properties':
+      tab_content.find('.journal').show();
+      tab_content.find('.journal:not(.has-details)').hide();
+      tab_content.find('.journal .wiki').hide();
+      tab_content.find('.journal .thumbnails').hide();
+      tab_content.find('.journal .contextual .journal-actions').hide();
+      break;
+    default:
+      tab_content.find('.journal').show();
+      tab_content.find('.journal .wiki').show();
+      tab_content.find('.journal .thumbnails').show();
+      tab_content.find('.journal .contextual .journal-actions').show();
+  }
+
+  return false;
+}
+
+function getRemoteTab(name, remote_url, url, load_always) {
+  load_always = load_always || false;
+  var tab_content = $('#tab-content-' + name);
+
+  tab_content.parent().find('.tab-content').hide();
+  tab_content.parent().children('div.tabs').find('a').removeClass('selected');
+  $('#tab-' + name).addClass('selected');
+
+  replaceInHistory(url);
+
+  if (tab_content.children().length == 0 && load_always == false) {
+    $.ajax({
+      url: remote_url,
+      type: 'get',
+      success: function(data){
+        tab_content.html(data)
+      }
+    });
+  }
+
+  tab_content.show();
+  return false;
+}
+
+//replaces current URL with the "href" attribute of the current link
+//(only triggered if supported by browser)
+function replaceInHistory(url) {
+  if ("replaceState" in window.history && url !== undefined) {
     window.history.replaceState(null, document.title, url);
   }
-  return false;
 }
 
 function moveTabRight(el) {
@@ -403,7 +480,7 @@ function displayTabsButtons() {
         numHidden++;
       }
     });
-    var bw = $(el).parents('div.tabs-buttons').outerWidth(true);
+    var bw = $(el).find('div.tabs-buttons').outerWidth(true);
     if ((tabsWidth < el.width() - bw) && (lis.length === 0 || lis.first().is(':visible'))) {
       el.find('div.tabs-buttons').hide();
     } else {
@@ -473,10 +550,12 @@ function scmEntryClick(id, url) {
     var el = $('#'+id);
     if (el.hasClass('open')) {
         collapseScmEntry(id);
+        el.find('.expander').switchClass('icon-expanded', 'icon-collapsed');
         el.addClass('collapsed');
         return false;
     } else if (el.hasClass('loaded')) {
         expandScmEntry(id);
+        el.find('.expander').switchClass('icon-collapsed', 'icon-expanded');
         el.removeClass('collapsed');
         return false;
     }
@@ -489,6 +568,7 @@ function scmEntryClick(id, url) {
       success: function(data) {
         el.after(data);
         el.addClass('open').addClass('loaded').removeClass('loading');
+        el.find('.expander').switchClass('icon-collapsed', 'icon-expanded');
       }
     });
     return true;
@@ -503,6 +583,23 @@ function randomKey(size) {
   return key;
 }
 
+function copyTextToClipboard(target) {
+  if (target) {
+    var temp = document.createElement('textarea');
+    temp.value = target.getAttribute('data-clipboard-text');
+    document.body.appendChild(temp);
+    temp.select();
+    document.execCommand('copy');
+    if (temp.parentNode) {
+      temp.parentNode.removeChild(temp);
+    }
+    if ($(target).closest('.drdn.expanded').length) {
+      $(target).closest('.drdn.expanded').removeClass("expanded");
+    }
+  }
+  return false;
+}
+
 function updateIssueFrom(url, el) {
   $('#all_attributes input, #all_attributes textarea, #all_attributes select').each(function(){
     $(this).data('valuebeforeupdate', $(this).val());
@@ -550,6 +647,46 @@ function observeAutocompleteField(fieldId, url, options) {
   });
 }
 
+function multipleAutocompleteField(fieldId, url, options) {
+  function split(val) {
+    return val.split(/,\s*/);
+  }
+
+  function extractLast(term) {
+    return split(term).pop();
+  }
+
+  $(document).ready(function () {
+    $('#' + fieldId).autocomplete($.extend({
+      source: function (request, response) {
+        $.getJSON(url, {
+          term: extractLast(request.term)
+        }, response);
+      },
+      minLength: 2,
+      position: {collision: "flipfit"},
+      search: function () {
+        $('#' + fieldId).addClass('ajax-loading');
+      },
+      response: function () {
+        $('#' + fieldId).removeClass('ajax-loading');
+      },
+      select: function (event, ui) {
+        var terms = split(this.value);
+        // remove the current input
+        terms.pop();
+        // add the selected item
+        terms.push(ui.item.value);
+        // add placeholder to get the comma-and-space at the end
+        terms.push("");
+        this.value = terms.join(", ");
+        return false;
+      }
+    }, options));
+    $('#' + fieldId).addClass('autocomplete');
+  });
+}
+
 function observeSearchfield(fieldId, targetId, url) {
   $('#'+fieldId).each(function() {
     var $this = $(this);
@@ -585,7 +722,7 @@ $(document).ready(function(){
 
   // This variable is used to focus selected project
   var selected;
-  $(".drdn-trigger").click(function(e){
+  $(document).on('click', '.drdn-trigger', function(e){
     var drdn = $(this).closest(".drdn");
     if (drdn.hasClass("expanded")) {
       drdn.removeClass("expanded");
@@ -659,12 +796,12 @@ function beforeShowDatePicker(input, inst) {
   var default_date = null;
   switch ($(input).attr("id")) {
     case "issue_start_date" :
-      if ($("#issue_due_date").size() > 0) {
+      if ($("#issue_due_date").length > 0) {
         default_date = $("#issue_due_date").val();
       }
       break;
     case "issue_due_date" :
-      if ($("#issue_start_date").size() > 0) {
+      if ($("#issue_start_date").length > 0) {
         var start_date = $("#issue_start_date").val();
         if (start_date != "") {
           start_date = new Date(Date.parse(start_date));
@@ -784,6 +921,21 @@ function setupFilePreviewNavigation() {
   }
 }
 
+$(document).on('keydown', 'form textarea', function(e) {
+  // Submit the form with Ctrl + Enter or Command + Return
+  var targetForm = $(e.target).closest('form');
+  if(e.keyCode == 13 && ((e.ctrlKey && !e.metaKey) || (!e.ctrlKey && e.metaKey) && targetForm.length)) {
+    // For ajax, use click() instead of submit() to prevent "Invalid form authenticity token" error
+    if (targetForm.attr('data-remote') == 'true') {
+      if (targetForm.find('input[type=submit]').length === 0) { return false; }
+      targetForm.find('textarea').blur().removeData('changed');
+      targetForm.find('input[type=submit]').first().click();
+    } else {
+      targetForm.find('textarea').blur().removeData('changed');
+      targetForm.submit();
+    }
+  }
+});
 
 
 function hideOnLoad() {
@@ -824,6 +976,15 @@ function toggleDisabledOnChange() {
 function toggleDisabledInit() {
   $('input[data-disables], input[data-enables], input[data-shows]').each(toggleDisabledOnChange);
 }
+function toggleMultiSelectIconInit() {
+  $('.toggle-multiselect:not(.icon-toggle-minus), .toggle-multiselect:not(.icon-toggle-plus)').each(function(){
+    if ($(this).siblings('select').find('option:selected').length > 1){
+      $(this).addClass('icon-toggle-minus');
+    } else {
+      $(this).addClass('icon-toggle-plus');
+    }
+  });
+}
 
 function toggleNewObjectDropdown() {
   var dropdown = $('#new-object + ul.menu-children');
@@ -863,6 +1024,17 @@ function toggleNewObjectDropdown() {
 $(document).ready(function(){
   $('#content').on('change', 'input[data-disables], input[data-enables], input[data-shows]', toggleDisabledOnChange);
   toggleDisabledInit();
+
+  $('#content').on('click', '.toggle-multiselect', function() {
+    toggleMultiSelect($(this).siblings('select'));
+    $(this).toggleClass('icon-toggle-plus icon-toggle-minus');
+  });
+  toggleMultiSelectIconInit();
+
+  $('#history .tabs').on('click', 'a', function(e){
+    var tab = $(e.target).attr('id').replace('tab-','');
+    document.cookie = 'history_last_tab=' + tab + '; SameSite=Lax'
+  });
 });
 
 $(document).ready(function(){
@@ -882,6 +1054,7 @@ $(document).ready(function(){
       data: "text=" + element + '&' + attachments,
       success: function(data){
         jstBlock.find('.wiki-preview').html(data);
+        setupWikiTableSortableHeader();
       }
     });
   });
@@ -907,7 +1080,7 @@ $(function ($) {
 function setFilecontentContainerHeight() {
   var $filecontainer = $('.filecontent-container');
   var fileTypeSelectors = ['.image', 'video'];
-  
+
   if($filecontainer.length > 0 && $filecontainer.find(fileTypeSelectors.join(',')).length === 1) {
     var containerOffsetTop = $filecontainer.offset().top;
     var containerMarginBottom = parseInt($filecontainer.css('marginBottom'));
@@ -923,6 +1096,130 @@ function setupAttachmentDetail() {
   $(window).resize(setFilecontentContainerHeight);
 }
 
+function setupWikiTableSortableHeader() {
+  $('div.wiki table').each(function(i, table){
+    if (table.rows.length < 3) return true;
+    var tr = $(table.rows).first();
+    if (tr.find("TH").length > 0) {
+      tr.attr('data-sort-method', 'none');
+      tr.find("TD").attr('data-sort-method', 'none');
+      new Tablesort(table);
+    }
+  });
+}
+
+$(function () {
+  $("[title]:not(.no-tooltip)").tooltip({
+    show: {
+      delay: 400
+    },
+    position: {
+      my: "center bottom-5",
+      at: "center top"
+    }
+  });
+});
+
+function inlineAutoComplete(element) {
+    'use strict';
+
+    // do not attach if Tribute is already initialized
+    if (element.dataset.tribute === 'true') {return};
+
+    const getDataSource = function(entity) {
+      const dataSources = rm.AutoComplete.dataSources;
+
+      if (dataSources[entity]) {
+        return dataSources[entity];
+      } else {
+        return false;
+      }
+    }
+
+    const remoteSearch = function(url, cb) {
+      const xhr = new XMLHttpRequest();
+      xhr.onreadystatechange = function ()
+      {
+        if (xhr.readyState === 4) {
+          if (xhr.status === 200) {
+            var data = JSON.parse(xhr.responseText);
+            cb(data);
+          } else if (xhr.status === 403) {
+            cb([]);
+          }
+        }
+      };
+      xhr.open("GET", url, true);
+      xhr.send();
+    };
+
+    const tribute = new Tribute({
+      collection: [
+        {
+          trigger: '#',
+          values: function (text, cb) {
+            if (event.target.type === 'text' && $(element).attr('autocomplete') != 'off') {
+              $(element).attr('autocomplete', 'off');
+            }
+            remoteSearch(getDataSource('issues') + text, function (issues) {
+              return cb(issues);
+            });
+          },
+          lookup: 'label',
+          fillAttr: 'label',
+          requireLeadingSpace: true,
+          selectTemplate: function (issue) {
+            return '#' + issue.original.id;
+          },
+          menuItemTemplate: function (issue) {
+            return sanitizeHTML(issue.original.label);
+          }
+        },
+        {
+          trigger: '[[',
+          values: function (text, cb) {
+            remoteSearch(getDataSource('wiki_pages') + text, function (wikiPages) {
+              return cb(wikiPages);
+            });
+          },
+          lookup: 'label',
+          fillAttr: 'label',
+          requireLeadingSpace: true,
+          selectTemplate: function (wikiPage) {
+            return '[[' + wikiPage.original.value + ']]';
+          },
+          menuItemTemplate: function (wikiPage) {
+            return sanitizeHTML(wikiPage.original.label);
+          }
+        },
+        {
+          trigger: '@',
+          lookup: function (user, mentionText) {
+            return user.name + user.firstname + user.lastname + user.login;
+          },
+          values: function (text, cb) {
+            const url = getDataSource('users');
+            if (url) {
+              remoteSearch(url + text, function (users) {
+                return cb(users);
+              });
+            }
+          },
+          menuItemTemplate: function (user) {
+            return user.original.name;
+          },
+          selectTemplate: function (user) {
+            return '@' + user.original.login;
+          }
+        }
+      ],
+      noMatchTemplate: ""
+    });
+
+    tribute.attach(element);
+}
+
+
 $(document).ready(setupAjaxIndicator);
 $(document).ready(hideOnLoad);
 $(document).ready(addFormObserversForDoubleSubmit);
@@ -930,3 +1227,7 @@ $(document).ready(defaultFocus);
 $(document).ready(setupAttachmentDetail);
 $(document).ready(setupTabs);
 $(document).ready(setupFilePreviewNavigation);
+$(document).ready(setupWikiTableSortableHeader);
+$(document).on('focus', '[data-auto-complete=true]', function(event) {
+  inlineAutoComplete(event.target);
+});
