@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -37,9 +39,7 @@ class EnumerationsControllerTest < Redmine::ControllerTest
   end
 
   def test_new
-    get :new, :params => {
-        :type => 'IssuePriority'
-      }
+    get(:new, :params => {:type => 'IssuePriority'})
     assert_response :success
 
     assert_select 'input[name=?][value=?]', 'enumeration[type]', 'IssuePriority'
@@ -47,20 +47,21 @@ class EnumerationsControllerTest < Redmine::ControllerTest
   end
 
   def test_new_with_invalid_type_should_respond_with_404
-    get :new, :params => {
-        :type => 'UnknownType'
-      }
+    get(:new, :params => {:type => 'UnknownType'})
     assert_response 404
   end
 
   def test_create
     assert_difference 'IssuePriority.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :enumeration => {
             :type => 'IssuePriority',
             :name => 'Lowest'
           }
         }
+      )
     end
     assert_redirected_to '/enumerations'
     e = IssuePriority.find_by_name('Lowest')
@@ -70,13 +71,16 @@ class EnumerationsControllerTest < Redmine::ControllerTest
   def test_create_with_custom_field_values
     custom_field = TimeEntryActivityCustomField.generate!
     assert_difference 'TimeEntryActivity.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :enumeration => {
             :type => 'TimeEntryActivity',
             :name => 'Sample',
             :custom_field_values => {custom_field.id.to_s => "sample"}
           }
         }
+      )
     end
     assert_redirected_to '/enumerations'
     assert_equal "sample", Enumeration.find_by(:name => 'Sample').custom_field_values.last.value
@@ -85,13 +89,16 @@ class EnumerationsControllerTest < Redmine::ControllerTest
   def test_create_with_multiple_select_list_custom_fields
     custom_field = IssuePriorityCustomField.generate!(:field_format => 'list', :multiple => true, :possible_values => ['1', '2', '3', '4'])
     assert_difference 'IssuePriority.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :enumeration => {
             :type => 'IssuePriority',
             :name => 'Sample',
             :custom_field_values => {custom_field.id.to_s => ['1', '2']}
           }
         }
+      )
     end
     assert_redirected_to '/enumerations'
     assert_equal ['1', '2'].sort, Enumeration.find_by(:name => 'Sample').custom_field_values.last.value.sort
@@ -99,41 +106,43 @@ class EnumerationsControllerTest < Redmine::ControllerTest
 
   def test_create_with_failure
     assert_no_difference 'IssuePriority.count' do
-      post :create, :params => {
+      post(
+        :create,
+        :params => {
           :enumeration => {
             :type => 'IssuePriority',
             :name => ''
           }
         }
+      )
     end
     assert_response :success
     assert_select_error /name cannot be blank/i
   end
 
   def test_edit
-    get :edit, :params => {
-        :id => 6
-      }
+    get(:edit, :params => {:id => 6})
     assert_response :success
     assert_select 'input[name=?][value=?]', 'enumeration[name]', 'High'
   end
 
   def test_edit_invalid_should_respond_with_404
-    get :edit, :params => {
-        :id => 999
-      }
+    get(:edit, :params => {:id => 999})
     assert_response 404
   end
 
   def test_update
     assert_no_difference 'IssuePriority.count' do
-      put :update, :params => {
+      put(
+        :update,
+        :params => {
           :id => 6,
           :enumeration => {
             :type => 'IssuePriority',
             :name => 'New name'
           }
         }
+      )
     end
     assert_redirected_to '/enumerations'
     e = IssuePriority.find(6)
@@ -142,13 +151,16 @@ class EnumerationsControllerTest < Redmine::ControllerTest
 
   def test_update_with_failure
     assert_no_difference 'IssuePriority.count' do
-      put :update, :params => {
+      put(
+        :update,
+        :params => {
           :id => 6,
           :enumeration => {
             :type => 'IssuePriority',
             :name => ''
           }
         }
+      )
     end
     assert_response :success
     assert_select_error /name cannot be blank/i
@@ -156,12 +168,15 @@ class EnumerationsControllerTest < Redmine::ControllerTest
 
   def test_update_position
     assert_equal 2, Enumeration.find(2).position
-    put :update, :params => {
-          :id => 2,
-          :enumeration => {
-            :position => 1,
+    put(
+      :update,
+      :params => {
+        :id => 2,
+        :enumeration => {
+          :position => 1,
         }
       }
+    )
     assert_response 302
     assert_equal 1, Enumeration.find(2).position
   end
@@ -170,21 +185,22 @@ class EnumerationsControllerTest < Redmine::ControllerTest
     custom_field = TimeEntryActivityCustomField.generate!
     enumeration = Enumeration.find(9)
     assert_nil enumeration.custom_field_values.last.value
-    put :update, :params => {
-          :id => enumeration.id,
-          :enumeration => {
-            :custom_field_values => {custom_field.id.to_s => "sample"}
+    put(
+      :update,
+      :params => {
+        :id => enumeration.id,
+        :enumeration => {
+          :custom_field_values => {custom_field.id.to_s => "sample"}
         }
       }
+    )
     assert_response 302
     assert_equal "sample", enumeration.reload.custom_field_values.last.value
   end
 
   def test_destroy_enumeration_not_in_use
     assert_difference 'IssuePriority.count', -1 do
-      delete :destroy, :params => {
-          :id => 7
-        }
+      delete(:destroy, :params => {:id => 7})
     end
     assert_redirected_to :controller => 'enumerations', :action => 'index'
     assert_nil Enumeration.find_by_id(7)
@@ -192,9 +208,7 @@ class EnumerationsControllerTest < Redmine::ControllerTest
 
   def test_destroy_enumeration_in_use
     assert_no_difference 'IssuePriority.count' do
-      delete :destroy, :params => {
-          :id => 4
-        }
+      delete(:destroy, :params => {:id => 4})
     end
     assert_response :success
 
@@ -207,10 +221,7 @@ class EnumerationsControllerTest < Redmine::ControllerTest
   def test_destroy_enumeration_in_use_with_reassignment
     issue = Issue.where(:priority_id => 4).first
     assert_difference 'IssuePriority.count', -1 do
-      delete :destroy, :params => {
-          :id => 4,
-          :reassign_to_id => 6
-        }
+      delete(:destroy, :params => {:id => 4, :reassign_to_id => 6})
     end
     assert_redirected_to :controller => 'enumerations', :action => 'index'
     assert_nil Enumeration.find_by_id(4)
@@ -220,10 +231,7 @@ class EnumerationsControllerTest < Redmine::ControllerTest
 
   def test_destroy_enumeration_in_use_with_blank_reassignment
     assert_no_difference 'IssuePriority.count' do
-      delete :destroy, :params => {
-          :id => 4,
-          :reassign_to_id => ''
-        }
+      delete(:destroy, :params => {:id => 4, :reassign_to_id => ''})
     end
     assert_response :success
   end
