@@ -1,5 +1,7 @@
+# frozen_string_literal: true
+
 # Redmine - project management software
-# Copyright (C) 2006-2017  Jean-Philippe Lang
+# Copyright (C) 2006-2022  Jean-Philippe Lang
 #
 # This program is free software; you can redistribute it and/or
 # modify it under the terms of the GNU General Public License
@@ -25,7 +27,7 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
            :issue_categories, :enumerations, :custom_fields, :custom_values, :trackers
 
   PRJ_ID = 3
-  NUM_REV = 11
+  NUM_REV = 13
 
   def setup
     super
@@ -42,10 +44,13 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
     def test_new
       @request.session[:user_id] = 1
       @project.repository.destroy
-      get :new, :params => {
+      get(
+        :new,
+        :params => {
           :project_id => 'subproject1',
           :repository_scm => 'Subversion'
         }
+      )
       assert_response :success
       assert_select 'select[name=?]', 'repository_scm' do
         assert_select 'option[value=?][selected=selected]', 'Subversion'
@@ -57,9 +62,12 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :show, :params => {
+      get(
+        :show,
+        :params => {
           :id => PRJ_ID
         }
+      )
       assert_response :success
 
       assert_select 'table.entries tbody' do
@@ -70,7 +78,7 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
 
       assert_select 'table.changesets tbody' do
         assert_select 'tr', 10
-        assert_select 'tr td.id a', :text => '11'
+        assert_select 'tr td.id a', :text => '12'
       end
 
       assert_select 'input[name=rev]'
@@ -83,11 +91,13 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       Repository::Subversion.create(:project => @project,
         :url => self.class.subversion_repository_url,
         :is_default => false, :identifier => 'svn')
-
-      get :show, :params => {
+      get(
+        :show,
+        :params => {
           :id => PRJ_ID,
           :repository_id => 'svn'
         }
+      )
       assert_response :success
 
       assert_select 'tr.dir a[href="/projects/subproject1/repository/svn/show/subversion_test"]'
@@ -100,11 +110,14 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :show, :params => {
+      get(
+        :show,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test'])[:param]
         }
+      )
       assert_response :success
 
       assert_select 'table.entries tbody' do
@@ -124,12 +137,15 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :show, :params => {
+      get(
+        :show,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test'])[:param],
           :rev => 4
         }
+      )
       assert_response :success
 
       assert_select 'table.entries tbody' do
@@ -147,11 +163,14 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :changes, :params => {
+      get(
+        :changes,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test', 'folder', 'helloworld.rb'])[:param]
         }
+      )
       assert_response :success
 
       assert_select 'table.changesets tbody' do
@@ -175,15 +194,19 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :changes, :params => {
+      get(
+        :changes,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test', 'folder'])[:param]
         }
+      )
       assert_response :success
-
       assert_select 'table.changesets tbody' do
-        assert_select 'tr', 6
+        assert_select 'tr', 8
+        assert_select 'tr td.id a', :text => '13'
+        assert_select 'tr td.id a', :text => '12'
         assert_select 'tr td.id a', :text => '10'
         assert_select 'tr td.id a', :text => '9'
         assert_select 'tr td.id a', :text => '7'
@@ -198,11 +221,14 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :entry, :params => {
+      get(
+        :entry,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test', 'helloworld.c'])[:param]
         }
+      )
       assert_response :success
       assert_select 'h2 a', :text => 'subversion_test'
       assert_select 'h2 a', :text => 'helloworld.c'
@@ -215,38 +241,86 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       assert_equal NUM_REV, @repository.changesets.count
       # no files in the test repo is larger than 1KB...
       with_settings :file_max_size_displayed => 0 do
-        get :entry, :params => {
+        get(
+          :entry,
+          :params => {
             :id => PRJ_ID,
             :repository_id => @repository.id,
             :path => repository_path_hash(['subversion_test', 'helloworld.c'])[:param]
           }
+        )
         assert_response :success
-        assert_equal 'text/html', @response.content_type
+        assert_equal 'text/html', @response.media_type
         assert_select 'p.nodata'
       end
     end
 
     def test_entry_should_display_images
-      get :entry, :params => {
+      get(
+        :entry,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test', 'folder', 'subfolder', 'rubylogo.gif'])[:param]
         }
+      )
       assert_response :success
       assert_select 'img[src=?]', "/projects/subproject1/repository/#{@repository.id}/raw/subversion_test/folder/subfolder/rubylogo.gif"
     end
 
+    def test_entry_should_preview_audio
+      get(
+        :entry,
+        :params => {
+          :id => PRJ_ID,
+          :repository_id => @repository.id,
+          :path => repository_path_hash(['subversion_test', 'folder', 'subfolder', 'chords.mp3'])[:param]
+        }
+      )
+      assert_response :success
+      assert_select 'audio[src=?]', "/projects/subproject1/repository/#{@repository.id}/raw/subversion_test/folder/subfolder/chords.mp3"
+    end
+
+    def text_entry_should_preview_markdown
+      get(
+        :entry,
+        :params => {
+          :id => PRJ_ID,
+          :repository_id => @repository.id,
+          :path => repository_path_hash(['subversion_test', 'folder', 'subfolder', 'testfile.md'])[:param]
+        }
+      )
+      assert_response :success
+      assert_select 'div.wiki', :html => "<h1>Header 1</h1>\n\n<h2>Header 2</h2>\n\n<h3>Header 3</h3>"
+    end
+
+    def text_entry_should_preview_textile
+      get(
+        :entry,
+        :params => {
+          :id => PRJ_ID,
+          :repository_id => @repository.id,
+          :path => repository_path_hash(['subversion_test', 'folder', 'subfolder', 'testfile.textile'])[:param]
+        }
+      )
+      assert_response :success
+      assert_select 'div.wiki', :html => "<h1>Header 1</h1>\n\n\n\t<h2>Header 2</h2>\n\n\n\t<h3>Header 3</h3>"
+    end
+
     def test_entry_at_given_revision
       assert_equal 0, @repository.changesets.count
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :entry, :params => {
+      get(
+        :entry,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test', 'helloworld.rb'])[:param],
           :rev => 2
         }
+      )
       assert_response :success
       # this line was removed in r3 and file was moved in r6
       assert_select 'td.line-code', :text => /Here's the code/
@@ -257,11 +331,14 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :entry, :params => {
+      get(
+        :entry,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test', 'zzz.c'])[:param]
         }
+      )
       assert_select 'p#errorExplanation', :text => /The entry or revision was not found in the repository/
     end
 
@@ -270,13 +347,16 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :raw, :params => {
+      get(
+        :raw,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test', 'helloworld.c'])[:param]
         }
+      )
       assert_response :success
-      assert_equal 'attachment; filename="helloworld.c"', @response.headers['Content-Disposition']
+      assert_equal "attachment; filename=\"helloworld.c\"; filename*=UTF-8''helloworld.c", @response.headers['Content-Disposition']
     end
 
     def test_directory_entry
@@ -284,11 +364,14 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :entry, :params => {
+      get(
+        :entry,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test', 'folder'])[:param]
         }
+      )
       assert_response :success
       assert_select 'h2 a', :text => 'subversion_test'
       assert_select 'h2 a', :text => 'folder'
@@ -296,11 +379,14 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
 
     # TODO: this test needs fixtures.
     def test_revision
-      get :revision, :params => {
+      get(
+        :revision,
+        :params => {
           :id => 1,
           :repository_id => 10,
           :rev => 2
         }
+      )
       assert_response :success
 
       assert_select 'ul' do
@@ -318,22 +404,28 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :revision, :params => {
+      get(
+        :revision,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :rev => 'something_weird'
         }
+      )
       assert_response 404
       assert_select_error /was not found/
     end
 
     def test_invalid_revision_diff
-      get :diff, :params => {
+      get(
+        :diff,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :rev => '1',
           :rev_to => 'something_weird'
         }
+      )
       assert_response 404
       assert_select_error /was not found/
     end
@@ -344,11 +436,14 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
       ['', ' ', nil].each do |r|
-        get :revision, :params => {
+        get(
+          :revision,
+          :params => {
             :id => PRJ_ID,
             :repository_id => @repository.id,
             :rev => r
           }
+        )
         assert_response 404
         assert_select_error /was not found/
       end
@@ -359,12 +454,14 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       r = Project.find(1).repository
       # Changes repository url to a subdirectory
       r.update_attribute :url, (r.url + '/test/some')
-
-      get :revision, :params => {
+      get(
+        :revision,
+        :params => {
           :id => 1,
           :repository_id => 10,
           :rev => 2
         }
+      )
       assert_response :success
 
       assert_select 'ul' do
@@ -383,12 +480,15 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
       ['inline', 'sbs'].each do |dt|
-        get :diff, :params => {
+        get(
+          :diff,
+          :params => {
             :id => PRJ_ID,
             :repository_id => @repository.id,
             :rev => 3,
             :type => dt
           }
+        )
         assert_response :success
         assert_select 'h2', :text => /Revision 3/
         assert_select 'th.filename', :text => 'subversion_test/textfile.txt'
@@ -400,15 +500,17 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-
-      get :diff, :params => {
+      get(
+        :diff,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :rev => 5,
           :format => 'diff'
         }
+      )
       assert_response :success
-      assert_equal 'text/x-patch', @response.content_type
+      assert_equal 'text/x-patch', @response.media_type
       assert_equal 'Index: subversion_test/folder/greeter.rb', @response.body.split(/\r?\n/).first
     end
 
@@ -418,7 +520,9 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
       ['inline', 'sbs'].each do |dt|
-        get :diff, :params => {
+        get(
+          :diff,
+          :params => {
             :id => PRJ_ID,
             :repository_id => @repository.id,
             :rev => 6,
@@ -426,6 +530,7 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
             :path => repository_path_hash(['subversion_test', 'folder'])[:param],
             :type => dt
           }
+        )
         assert_response :success
 
         assert_select 'h2', :text => /2:6/
@@ -441,22 +546,25 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :annotate, :params => {
+      get(
+        :annotate,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :path => repository_path_hash(['subversion_test', 'helloworld.c'])[:param]
         }
+      )
       assert_response :success
 
       assert_select 'tr' do
-        assert_select 'th.line-num', :text => '1'
+        assert_select 'th.line-num a[data-txt=?]', '1'
         assert_select 'td.revision', :text => '4'
         assert_select 'td.author', :text => 'jp'
         assert_select 'td', :text => /stdio.h/
       end
       # Same revision
       assert_select 'tr' do
-        assert_select 'th.line-num', :text => '2'
+        assert_select 'th.line-num a[data-txt=?]', '2'
         assert_select 'td.revision', :text => ''
         assert_select 'td.author', :text => ''
       end
@@ -467,12 +575,15 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       @repository.fetch_changesets
       @project.reload
       assert_equal NUM_REV, @repository.changesets.count
-      get :annotate, :params => {
+      get(
+        :annotate,
+        :params => {
           :id => PRJ_ID,
           :repository_id => @repository.id,
           :rev => 8,
           :path => repository_path_hash(['subversion_test', 'helloworld.c'])[:param]
         }
+      )
       assert_response :success
       assert_select 'h2', :text => /@ 8/
     end
@@ -484,9 +595,7 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
       assert_equal NUM_REV, @repository.changesets.count
 
       assert_difference 'Repository.count', -1 do
-        delete :destroy, :params => {
-            :id => @repository.id
-          }
+        delete(:destroy, :params => {:id => @repository.id})
       end
       assert_response 302
       @project.reload
@@ -496,16 +605,17 @@ class RepositoriesSubversionControllerTest < Redmine::RepositoryControllerTest
     def test_destroy_invalid_repository
       @request.session[:user_id] = 1 # admin
       @project.repository.destroy
-      @repository = Repository::Subversion.create!(
-                       :project => @project,
-                       :url     => "file:///invalid")
+      @repository =
+        Repository::Subversion.
+          create!(
+            :project => @project,
+            :url     => "file:///invalid"
+          )
       @repository.fetch_changesets
       assert_equal 0, @repository.changesets.count
 
       assert_difference 'Repository.count', -1 do
-        delete :destroy, :params => {
-            :id => @repository.id
-          }
+        delete(:destroy, :params => {:id => @repository.id})
       end
       assert_response 302
       @project.reload
